require('https').globalAgent.options.ca = require('ssl-root-cas/latest').create();
require('ssl-root-cas').inject();
process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";

import Logging from "@hibas123/nodelogging";
import * as fs from "fs";

import * as dotenv from "dotenv";
dotenv.config();
import rss from "rss-parser";
const parser = new rss();

import fetch from "node-fetch";
import { createHash } from "crypto";
import low from "lowdb";
import FileSync from "lowdb/adapters/FileSync";

if (!fs.existsSync("./persist")) {
   fs.mkdirSync("./persist")
}

const adapter = new FileSync<{ test }>("persist/db.json");
const db = low(adapter);
db.defaults({});
import { createTransport } from "nodemailer";

let transporter = createTransport({
   host: process.env.MAIL_SERVER,
   port: 587,
   secure: false, // true for 465, false for other ports
   auth: {
      user: process.env.MAIL_USERNAME,
      pass: process.env.MAIL_PASSWORD
   }
});

interface Feed {
   title: string;
   language: string;
   description: string;
   copyright: string;

   items: FeedItem[];
}

interface FeedItem {
   title: string;
   link: string;
   content: string,
   contentSnippet: string;
   guid: string;
}

function calculateHash(item: FeedItem) {
   let hash = createHash("sha512");
   hash.update(item.content + item.guid + item.title);
   return hash.digest("hex");
}

async function check() {
   try {
      let data = await fetch(process.env.FEED).then(res => res.text());
      Logging.log("Received Data");
      let feed: Feed = await parser.parseString(data).catch(err => Logging.error(err)) as any;
      //Check for new entries
      let items = feed.items.filter(item => {
         return db.get(calculateHash(item)).value() !== true;
      })
      for (let item of items) {
         await sendFeedMail(feed, item).catch(err => Logging.error(err))
         await sendFeedTelegraf(feed, item).catch(err => Logging.error(err));
      }
   } catch (error) {
      Logging.error(error);
   }
}

import Telegraf from "telegraf";
const bot = new Telegraf(process.env.TG_TOKEN)
const chatidFilename = "./persist/chatid";
let chatid: number;
try {
   if (fs.existsSync(chatidFilename)) {
      chatid = Number(fs.readFileSync(chatidFilename).toString());
   }
} catch (err) {
   Logging.error(err);
}

bot.start(ctx => {
   let code = ctx.message.text.split(" ")[1];
   if (code === process.env.TG_CODE) {
      if (chatid) {
         bot.telegram.sendMessage(chatid, "This chat is now disabled!");
      }
      chatid = ctx.chat.id;
      fs.writeFileSync(chatidFilename, chatid.toString());
      ctx.reply("News enabled!");
   } else {
      ctx.reply("Invalid code!");
   }
})

bot.startPolling();

async function sendFeedTelegraf(feed: Feed, item: FeedItem) {
   if (chatid) {
      await bot.telegram.sendMessage(chatid, entities.decode(item.title) + "\n\n" + entities.decode(item.contentSnippet));
   }
}

import { AllHtmlEntities } from "html-entities";
import { fstat } from "fs";
const entities = new AllHtmlEntities();
async function sendFeedMail(feed: Feed, item: FeedItem) {
   Logging.log("Sending Mail!")
   let mailOptions = {
      from: process.env.MAIL_SENDER,
      to: process.env.MAIL_RECEIVER,
      subject: '[OSPLUS_UPDATE] ' + entities.decode(item.title),
      text: entities.decode(item.contentSnippet),
      html: item.content
   };

   let info = await transporter.sendMail(mailOptions);
   Logging.log('Message sent: %s', info.messageId);
   db.set(calculateHash(item), true).write();
}

setInterval(() => check(), 1000 * 60 * 5);
check();